@@ -11,10 +11,7 @@ import codecs
 
 to_utf8 = codecs.getencoder('utf-8')
 
-FALSE = g.FALSE
-TRUE = g.TRUE
-
-from rox.Menu import Menu, set_save_name
+from rox.Menu import Menu, set_save_name, SubMenu, Separator, Action, ToggleItem
 
 default_font = Option('default_font', 'serif')
 
@@ -35,29 +32,30 @@ layout_indent_para = Option('layout_indent_para', 2)
 set_save_name('Edit')
 
 menu = Menu('main', [
-(_('/File'),			'',	'<Branch>'),
-(_('/File/') + _('Save'),	'save',		'<StockItem>', '<Ctrl>S', g.STOCK_SAVE),
-(_('/File/') + _('Open Parent'),	'up',		'<StockItem>', '', g.STOCK_GO_UP),
-(_('/File/') + _('Show Changes'),	'diff', 	'<StockItem>', '', 'rox-diff'),
-(_('/File/') + _('Close'),	'close',	'<StockItem>', '', g.STOCK_CLOSE),
-(_('/File/'),			'',	'<Separator>'),
-(_('/File/') + _('New'),		'new',		'<StockItem>', '', g.STOCK_NEW),
+SubMenu(_('File'), [
+  Action(_('Save'),	'save',		'<Ctrl>S', g.STOCK_SAVE),
+  Action(_('Open Parent'),	'up',		'', g.STOCK_GO_UP),
+  Action(_('Show Changes'),	'diff', 	'', 'rox-diff'),
+  ToggleItem(_('Word Wrap'), 'word_wrap'),
+  Action(_('Close'),	'close',	'', g.STOCK_CLOSE),
+  Separator(),
+  Action(_('New'),	'new',		'', g.STOCK_NEW)]),
 
-(_('/Edit'),			'',	'<Branch>'),
-(_('/Edit/') + _('Cut'),		'cut',		'<StockItem>', '<Ctrl>X', g.STOCK_CUT),
-(_('/Edit/') + _('Copy'),	'copy',		'<StockItem>', '<Ctrl>C', g.STOCK_COPY),
-(_('/Edit/') + _('Paste'),	'paste',	'<StockItem>', '<Ctrl>V', g.STOCK_PASTE),
-(_('/Edit/'),			'',	'<Separator>'),
-(_('/Edit/') + _('Undo'),	'undo',		'<StockItem>', '<Ctrl>Z', g.STOCK_UNDO),
-(_('/Edit/') + _('Redo'),	'redo',		'<StockItem>', '<Ctrl>Y', g.STOCK_REDO),
-(_('/Edit/'),			'',	'<Separator>'),
-(_('/Edit/') + _('Search...'),	'search',	'<StockItem>', 'F4', g.STOCK_FIND),
-(_('/Edit/') + _('Search and Replace....'), 'search_replace',
-					'<StockItem>', '<Ctrl>F4', g.STOCK_FIND_AND_REPLACE),
-(_('/Edit/') + _('Goto line...'),	'goto',		'<StockItem>', 'F5', g.STOCK_JUMP_TO),
+SubMenu(_('Edit'), [
+  Action(_('Cut'),		'cut',		'<Ctrl>X', g.STOCK_CUT),
+  Action(_('Copy'),	'copy',		'<Ctrl>C', g.STOCK_COPY),
+  Action(_('Paste'),	'paste',	'<Ctrl>V', g.STOCK_PASTE),
+  Separator(),
+  Action(_('Undo'),	'undo',		'<Ctrl>Z', g.STOCK_UNDO),
+  Action(_('Redo'),	'redo',		'<Ctrl>Y', g.STOCK_REDO),
+  Separator(),
+  Action(_('Search...'),	'search',	'F4', g.STOCK_FIND),
+  Action(_('Search and Replace....'), 'search_replace',
+					'<Ctrl>F4', g.STOCK_FIND_AND_REPLACE),
+  Action(_('Goto line...'),	'goto',		'F5', g.STOCK_JUMP_TO)]),
 
-(_('/Options'),			'show_options', '<StockItem>', '', g.STOCK_PROPERTIES),
-(_('/Help'),			'help',		'<StockItem>', 'F1', g.STOCK_HELP),
+Action(_('Options'),		'show_options', '', g.STOCK_PROPERTIES),
+Action(_('Help'),		'help',		'F1', g.STOCK_HELP),
 ])
 
 known_codecs = (
@@ -113,6 +111,9 @@ class DiffLoader(XDSLoader):
 		self.window.diff(path = tmp.name)
 
 class EditWindow(rox.Window, XDSLoader, Saveable):
+	_word_wrap = False
+	wrap_button = None
+	
 	def __init__(self, filename = None):
 		rox.Window.__init__(self)
 		XDSLoader.__init__(self, ['text/plain', 'UTF8_STRING'])
@@ -130,14 +131,13 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 		self.text.set_buffer(self.buffer)
 		self.text.set_size_request(10, 10)
 		self.xds_proxy_for(self.text)
-		self.update_styles()
 		
 		self.insert_mark = self.buffer.get_mark('insert')
 		self.selection_bound_mark = self.buffer.get_mark('selection_bound')
 		start = self.buffer.get_start_iter()
-		self.mark_start = self.buffer.create_mark('mark_start', start, TRUE)
-		self.mark_end = self.buffer.create_mark('mark_end', start, FALSE)
-		self.mark_tmp = self.buffer.create_mark('mark_tmp', start, FALSE)
+		self.mark_start = self.buffer.create_mark('mark_start', start, True)
+		self.mark_end = self.buffer.create_mark('mark_end', start, False)
+		self.mark_tmp = self.buffer.create_mark('mark_tmp', start, False)
 		tag = self.buffer.create_tag('marked')
 		tag.set_property('background', 'green')
 		self.marked = 0
@@ -145,14 +145,14 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 		# When searching, this is where the cursor was when the minibuffer
 		# was opened.
 		start = self.buffer.get_start_iter()
-		self.search_base = self.buffer.create_mark('search_base', start, TRUE)
+		self.search_base = self.buffer.create_mark('search_base', start, True)
 
-		vbox = g.VBox(FALSE)
+		vbox = g.VBox(False)
 		self.add(vbox)
 
 		tools = g.Toolbar()
 		tools.set_style(g.TOOLBAR_ICONS)
-		vbox.pack_start(tools, FALSE, TRUE, 0)
+		vbox.pack_start(tools, False, True, 0)
 		tools.show()
 
 		self.status_label = g.Label('')
@@ -162,6 +162,14 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 				'Or, drop a backup file onto this button to see changes from that.'),
 				None, self.diff, None, 0)
 		DiffLoader(self).xds_proxy_for(diff)
+
+		image_wrap = g.Image()
+		image_wrap.set_from_file(rox.app_dir + '/images/rox-word-wrap.png')
+		self.wrap_button = tools.insert_element(g.TOOLBAR_CHILD_TOGGLEBUTTON,
+					None, _("Word Wrap"), _("Word Wrap"), None,
+					image_wrap,
+					lambda button: self.set_word_wrap(button.get_active()),
+					None, 0)
 		tools.insert_stock(g.STOCK_REDO, _('Redo'), None, self.redo, None, 0)
 		tools.insert_stock(g.STOCK_UNDO, _('Undo'), None, self.undo, None, 0)
 		tools.insert_stock(g.STOCK_FIND_AND_REPLACE, _('Replace'), None, self.search_replace, None, 0)
@@ -178,10 +186,11 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 
 		swin.add(self.text)
 
+		self.update_styles()
 		self.show_all()
 
 		# Create the minibuffer
-		self.mini_hbox = g.HBox(FALSE)
+		self.mini_hbox = g.HBox(False)
 		info = g.Button()
 		info.set_relief(g.RELIEF_NONE)
 		info.unset_flags(g.CAN_FOCUS)
@@ -191,12 +200,12 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 		info.show_all()
 		info.connect('clicked', self.mini_show_info)
 		
-		self.mini_hbox.pack_start(info, FALSE, TRUE, 0)
+		self.mini_hbox.pack_start(info, False, True, 0)
 		self.mini_label = g.Label('')
-		self.mini_hbox.pack_start(self.mini_label, FALSE, TRUE, 0)
+		self.mini_hbox.pack_start(self.mini_label, False, True, 0)
 		self.mini_entry = g.Entry()
-		self.mini_hbox.pack_start(self.mini_entry, TRUE, TRUE, 0)
-		vbox.pack_start(self.mini_hbox, FALSE, TRUE)
+		self.mini_hbox.pack_start(self.mini_entry, True, True, 0)
+		vbox.pack_start(self.mini_hbox, False, True)
 		self.mini_entry.connect('key-press-event', self.mini_key_press)
 		self.mini_entry.connect('changed', self.mini_changed)
 
@@ -243,13 +252,12 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 				self.load_file(filename)
 				if filename != '-':
 					self.save_last_stat = os.stat(filename)
-					self.save_mode = self.save_last_stat.st_mode # for older ROX-Libs
 			except Abort:
 				self.destroy()
 				raise
 
 		self.buffer.connect('modified-changed', self.update_title)
-		self.buffer.set_modified(FALSE)
+		self.buffer.set_modified(False)
 
 		def button_press(text, event):
 			if event.button != 3:
@@ -302,10 +310,7 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 			self.text.set_pixels_inside_wrap(layout_inside_para.int_value)
 			self.text.set_indent(layout_indent_para.int_value)
 
-			if word_wrap.int_value == 1:
-				self.text.set_wrap_mode(g.WRAP_WORD)
-			else:
-				self.text.set_wrap_mode(g.WRAP_NONE)
+			self.word_wrap = bool(word_wrap.int_value)
 		except:
 			rox.report_exception()
 		else:
@@ -340,23 +345,23 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 	def get_encoding(self, message):
 		"Returns (encoding, errors), or raises Abort to cancel."
 		box = g.MessageDialog(self, 0, g.MESSAGE_QUESTION, g.BUTTONS_CANCEL, message)
-		box.set_has_separator(FALSE)
+		box.set_has_separator(False)
 
 		frame = g.Frame()
-		box.vbox.pack_start(frame, TRUE, TRUE)
+		box.vbox.pack_start(frame, True, True)
 		frame.set_border_width(6)
 
-		hbox = g.HBox(FALSE, 4)
+		hbox = g.HBox(False, 4)
 		hbox.set_border_width(6)
 
-		hbox.pack_start(g.Label('Encoding:'), FALSE, TRUE, 0)
+		hbox.pack_start(g.Label(_('Encoding:')), False, True, 0)
 		combo = g.Combo()
 		combo.disable_activate()
 		combo.entry.connect('activate', lambda w: box.activate_default())
 		combo.set_popdown_strings(known_codecs)
-		hbox.pack_start(combo, TRUE, TRUE, 0)
-		ignore_errors = g.CheckButton('Ignore errors')
-		hbox.pack_start(ignore_errors, FALSE, TRUE)
+		hbox.pack_start(combo, True, True, 0)
+		ignore_errors = g.CheckButton(_('Ignore errors'))
+		hbox.pack_start(ignore_errors, False, True)
 
 		frame.add(hbox)
 		
@@ -493,11 +498,11 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 	def save_to_stream(self, stream):
 		s = self.buffer.get_start_iter()
 		e = self.buffer.get_end_iter()
-                stream.write(self.buffer.get_text(s, e, TRUE))
+                stream.write(self.buffer.get_text(s, e, True))
 	
 	def set_uri(self, uri):
                 self.uri = uri
-		self.buffer.set_modified(FALSE)
+		self.buffer.set_modified(False)
                 self.update_title()
 	
 	def new(self):
@@ -643,6 +648,16 @@ class EditWindow(rox.Window, XDSLoader, Saveable):
 				start = self.buffer.get_iter_at_mark(self.mark_tmp)
 				end = get_end()
 			if not start.forward_line(): break
+
+	def set_word_wrap(self, value):
+		self._word_wrap = value
+		self.wrap_button.set_active(value)
+		if value:
+			self.text.set_wrap_mode(g.WRAP_WORD)
+		else:
+			self.text.set_wrap_mode(g.WRAP_NONE)
+	
+	word_wrap = property(lambda self: self._word_wrap, set_word_wrap)
 	
 class SelectionSaver(Saveable):
 	def __init__(self, window):
@@ -651,7 +666,7 @@ class SelectionSaver(Saveable):
 	
 	def save_to_stream(self, stream):
 		s, e = self.window.get_marked_range()
-		stream.write(self.window.buffer.get_text(s, e, TRUE))
+		stream.write(self.window.buffer.get_text(s, e, True))
 	
 	def destroy(self):
 		# Called when savebox is remove. Get rid of the selection marker
